#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
from .refactor.api import Api
from .pluins.token import Token
from .refactor.argument import Argument
from .refactor.app import Flask, FlaskInit
from .refactor.request_parser import RequestParser
from .refactor.http_code import (
    GainSuccess,
    CreateSuccess,
    UpdateSuccess,
    DeleteSuccess,
    ServerException,
    NotFoundException,
    ParameterException,
    ForbiddenException,
    BadRequestException,
    AuthFailedException,
)

__version__ = "0.2.1"



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
from .response import ApiResponse


class GainSuccess(ApiResponse):
    code = 200
    message = "Success"
    state_code = 1


class UpdateSuccess(ApiResponse):
    code = 200
    message = "Update success"
    state_code = 1


class CreateSuccess(ApiResponse):
    code = 201
    message = "Created success"
    state_code = 1


class DeleteSuccess(ApiResponse):
    code = 204
    message = "Deleted success"
    state_code = 1


class BadRequestException(ApiResponse):
    code = 400
    message = "Client error"
    state_code = 0


class AuthFailedException(ApiResponse):
    code = 401
    message = "Authentication failed"
    state_code = -1


class ForbiddenException(ApiResponse):
    code = 403
    message = "Forbidden"
    state_code = 0


class NotFoundException(ApiResponse):
    code = 404
    message = "NotFound"
    state_code = 0


class ParameterException(ApiResponse):
    code = 422
    message = "Parameter error"
    state_code = 0


class ServerException(ApiResponse):
    code = 500
    message = "Sorry, an unknown error occurred in the system. (*￣︶￣)!"
    state_code = 0



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
from __future__ import unicode_literals

import six
import sys

from flask_restx import Api as _Api
from flask_restx.utils import unpack
from flask import request, current_app
from flask_restx._http import HTTPStatus
from werkzeug.datastructures import Headers
from flask_restx.api import HEADERS_BLACKLIST
from flask.signals import got_request_exception
from flask import make_response as original_flask_make_response
from werkzeug.exceptions import HTTPException, NotAcceptable, InternalServerError

from .response import ApiResponse


class Api(_Api):

    def make_response(self, data, *args, **kwargs):
        """
        Looks up the representation transformer for the requested media
        type, invoking the transformer to create a response object. This
        defaults to default_mediatype if no transformer is found for the
        requested mediatype. If default_mediatype is None, a 406 Not
        Acceptable response will be sent as per RFC 2616 section 14.1

        :param data: Python object containing response data to be transformed
        :param data:
        :param args:
        :param kwargs:
        :return:
        """

        default_mediatype = kwargs.pop('fallback_mediatype', None) or self.default_mediatype
        mediatype = request.accept_mimetypes.best_match(
            self.representations,
            default=default_mediatype,
        )
        if mediatype is None:
            raise NotAcceptable()

        if isinstance(data, ApiResponse):
            return data

        if mediatype in self.representations:
            resp = self.representations[mediatype](data, *args, **kwargs)
            resp.headers['Content-Type'] = mediatype
            return resp
        elif mediatype == 'text/plain':
            resp = original_flask_make_response(str(data), *args, **kwargs)
            resp.headers['Content-Type'] = 'text/plain'
            return resp
        else:
            raise InternalServerError()

    def handle_error(self, e):
        """
        Error handler for the API transforms a raised exception into a Flask response,
        with the appropriate HTTP status code and body.

        :param Exception e: the raised Exception object
        :param e:
        :return:
        """
        if (
                not isinstance(e, HTTPException)
                and self._propagate_exceptions()
                and not isinstance(e, tuple(self._own_and_child_error_handlers.keys()))
        ):
            exc_type, exc_value, tb = sys.exc_info()
            if exc_value is e:
                raise
            else:
                raise e

        include_message_in_response = current_app.config.get(
            "ERROR_INCLUDE_MESSAGE", True
        )
        default_data = {}

        headers = Headers()

        for typecheck, handler in self._own_and_child_error_handlers.items():
            if isinstance(e, typecheck):
                result = handler(e)
                default_data, code, headers = unpack(
                    result, HTTPStatus.INTERNAL_SERVER_ERROR
                )
                break
        else:
            # Flask docs say: "This signal is not sent for HTTPException or other exceptions that have error handlers
            # registered, unless the exception was raised from an error handler."
            got_request_exception.send(current_app._get_current_object(), exception=e)

            if isinstance(e, HTTPException):
                code = None
                if e.code is not None:
                    code = HTTPStatus(e.code)
                elif e.response is not None:
                    code = HTTPStatus(e.response.status_code)
                if include_message_in_response:
                    default_data = {"message": e.description or code.phrase}
                headers = e.get_response().headers
            elif self._default_error_handler:
                result = self._default_error_handler(e)
                default_data, code, headers = unpack(
                    result, HTTPStatus.INTERNAL_SERVER_ERROR
                )
            else:
                code = HTTPStatus.INTERNAL_SERVER_ERROR
                if include_message_in_response:
                    default_data = {
                        "message": code.phrase,
                    }

        if include_message_in_response:
            default_data["message"] = default_data.get("message", str(e))

        data = getattr(e, "data", default_data)
        fallback_mediatype = None

        if code >= HTTPStatus.INTERNAL_SERVER_ERROR:
            exc_info = sys.exc_info()
            if exc_info[1] is None:
                exc_info = None
            current_app.log_exception(exc_info)

        elif (
                code == HTTPStatus.NOT_FOUND
                and current_app.config.get("RESTX_ERROR_404_HELP", True)
                and include_message_in_response
        ):
            data["message"] = self._help_on_404(data.get("message", None))

        elif code == HTTPStatus.NOT_ACCEPTABLE and self.default_mediatype is None:
            # if we are handling NotAcceptable (406), make sure that
            # make_response uses a representation we support as the
            # default mediatype (so that make_response doesn't throw
            # another NotAcceptable error).
            supported_mediatypes = list(self.representations.keys())
            fallback_mediatype = (
                supported_mediatypes[0] if supported_mediatypes else "text/plain"
            )

        # Remove blacklisted headers
        for header in HEADERS_BLACKLIST:
            headers.pop(header, None)
        if isinstance(e, ApiResponse):
            e.data = data
            resp = self.make_response(e, code, headers, fallback_mediatype=fallback_mediatype)
        else:
            resp = self.make_response(data, code, headers, fallback_mediatype=fallback_mediatype)

        if code == HTTPStatus.UNAUTHORIZED:
            resp = self.unauthorized(resp)
        return resp



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
from flask_restx.reqparse import Argument as _Argument, _handle_arg_type, LOCATIONS


class Argument(_Argument):

    @property
    def __schema__(self):
        if self.location == 'cookie':
            return
        param = {
            'name': self.name,
            'in': LOCATIONS.get(self.location[-1], 'args') if isinstance(self.location,
                                                                         (tuple, list)) else LOCATIONS.get(
                self.location, 'args')
        }
        _handle_arg_type(self, param)
        if self.required:
            param['required'] = True
        if self.help:
            param['description'] = self.help
        if self.default is not None:
            param['default'] = self.default() if callable(self.default) else self.default
        if self.action == 'append':
            param['items'] = {'type': param['type']}
            param['type'] = 'array'
            param['collectionFormat'] = 'multi'
        if self.action == 'split':
            param['items'] = {'type': param['type']}
            param['type'] = 'array'
            param['collectionFormat'] = 'csv'
        if self.choices:
            param['enum'] = self.choices
            param['collectionFormat'] = 'multi'
        return param



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-21
import uuid
import time
import decimal
import traceback
import dataclasses
from typing import Union, Type
from datetime import date, datetime

from flask import g
from flask import Flask as _Flask
from flask.json.provider import JSONProvider
from werkzeug.exceptions import HTTPException
from flask.json.provider import DefaultJSONProvider as _DefaultJSONProvider

from .api import Api
from .sqlalchemy import db
from .requester import Request
from .response import ApiResponse
from ..pluins.global_logger import GlobalLogger
from .http_code import ServerException


class DefaultJSONProvider(_DefaultJSONProvider):

    @staticmethod
    def default(o):
        if hasattr(o, "keys") and hasattr(o, "__getitem__"):
            return dict(o)

        if isinstance(o, date):
            return o.strftime("%Y-%m-%d")

        if isinstance(o, datetime):
            return o.strftime("%Y-%m-%d %H:%M:%S")

        if isinstance(o, (decimal.Decimal, uuid.UUID)):
            return str(o)

        if dataclasses and dataclasses.is_dataclass(o):
            return dataclasses.asdict(o)

        if hasattr(o, "__html__"):
            return str(o.__html__())

        raise TypeError(f"Object of type {type(o).__name__} is not JSON serializable")


class Flask(_Flask):
    request_class: Type[Request] = Request
    json_provider_class: Type[JSONProvider] = DefaultJSONProvider


class FlaskInit:
    """"""

    @classmethod
    def register_api(cls, app: Flask, api: Api, hook: bool = False):
        """"""
        api.init_app(app=app)

        if hook is True:
            cls.register_hook(app=app, api=api)
        return

    @classmethod
    def register_config(cls, app: Flask, configs: Union[str, list], sqlalchemy: bool = False,
                        create_all: bool = False) -> None:
        """"""
        if not isinstance(configs, list):
            configs = [configs]

        for config in configs:
            app.config.from_object(config)

        if sqlalchemy is True:
            cls.register_sqlalchemy(app=app, create_all=create_all)

        return

    @classmethod
    def register_sqlalchemy(cls, app: Flask, create_all: bool = False) -> None:
        """"""
        db.init_app(app)

        if create_all is True:
            with app.app_context():
                db.create_all()
        return

    @classmethod
    def register_hook(cls, app: Flask, api: Api) -> None:
        """"""

        @app.errorhandler(Exception)
        def error_handler(e):
            """"""
            if isinstance(e, ApiResponse):
                return e
            if isinstance(e, HTTPException):
                if e.code == 500:
                    state_code = -1
                else:
                    state_code = 0
                return ApiResponse(http_code=e.code, message=e.description, state_code=state_code)
            else:
                if app.config['DEBUG'] is True:
                    traceback.print_exc()
                    return ServerException()
                else:
                    return ServerException()

        @app.after_request
        def after_request(response):
            """"""
            content_type = response.headers.get("Content-Type")
            if "application/json" in content_type:
                GlobalLogger(response=response, api=api)
            return response

        @app.before_request
        def before_request():
            """"""
            g.request_time = int(time.time())

        return



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-23
import time

import jwt
from flask import g
from typing import Callable
from flask_httpauth import HTTPTokenAuth

from ..refactor.http_code import AuthFailedException


class Token:
    """"""

    def __init__(self, secret: str, scheme: str = "JWT", algorithm: str = "HS256", message: str = "认证失败"):
        """"""
        self.secret = secret
        self.message = message
        self.algorithm = algorithm
        self.permission_callback = None
        self.auth = HTTPTokenAuth(scheme=scheme)
        self.init_app()

    def init_app(self):
        """"""

        @self.auth.verify_token
        def verify_token(token: str):
            """"""
            try:
                data = jwt.decode(token, self.secret, algorithms=self.algorithm)
            except jwt.exceptions.ExpiredSignatureError:
                raise AuthFailedException(message=self.message)
            except jwt.exceptions.InvalidSignatureError:
                raise AuthFailedException(message=self.message)
            except jwt.exceptions.DecodeError:
                raise AuthFailedException(message=self.message)

            user = data.get("user", dict())

            if self.permission_callback is not None:
                self.permission_callback(user)

            g.user = user
            return data

        return self

    def generate_token(self, user: dict = None, expiration: int = None) -> str:
        """生成令牌"""
        payload = dict()
        if user is not None:
            payload["user"] = user

        if expiration is not None:
            iat = int(time.time())
            payload["iat"] = iat
            payload["exp"] = iat + expiration

        token = jwt.encode(payload=payload, key=self.secret, algorithm=self.algorithm)
        return token

    def verify_permission(self, func: Callable):
        self.permission_callback = func
        return func



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2024/4/23
import time
import random
import string
import unittest

from example.create_flask import flask_app as app


class TestFlaskApi(unittest.TestCase):
    """"""

    def setUp(self):
        """"""
        self.app = app.test_client()
        self.app.testing = True

    def tearDown(self):
        """"""
        time.sleep(5)

    @staticmethod
    def generate_random_string(min_length=5, max_length=10):
        """"""
        length = random.randint(min_length, max_length)
        letters = string.ascii_letters + string.digits
        random_string = ''.join(random.choice(letters) for _ in range(length))
        return random_string

    def generate_params(self):
        """"""
        return (
            random.randint(10000, 100000),
            self.generate_random_string(),
            self.generate_random_string(),
            self.generate_random_string()
        )

    def test_get(self):
        """"""
        pid, a, b, c = self.generate_params()
        compare = dict(pid=pid, a=a, b=b, c=c)
        response = self.app.get(f"/api/test/{pid}?a={a}&b={b}&c={c}")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json.get("data"), compare)

    def test_post(self):
        """"""
        pid, a, b, c = self.generate_params()
        compare = dict(pid=pid, a=a, b=b, c=c)
        response = self.app.post(f"/api/test/{pid}?c={c}", json={"a": a, "b": b})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.json.get("data"), compare)

    def test_patch(self):
        """"""
        pid, a, b, c = self.generate_params()
        compare = dict(pid=pid, a=a, b=b, c=c)
        response = self.app.patch(f"/api/test/{pid}?c={c}", json={"a": a, "b": b})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json.get("data"), compare)

    def test_put(self):
        """"""
        pid, a, b, c = self.generate_params()
        compare = dict(pid=pid, a=a, b=b, c=c)
        response = self.app.put(f"/api/test/{pid}?c={c}", json={"a": a, "b": b})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json.get("data"), compare)

    def test_delete(self):
        """"""
        pid, a, b, c = self.generate_params()
        response = self.app.delete(f"/api/test/{pid}?a={a}&b={b}&c={c}")
        self.assertEqual(response.status_code, 204)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# TIME ： 2022-07-23
import re
from setuptools import find_packages, setup

with open("fourcats_flask/__init__.py", "r") as file:
    regex_version = r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]'
    version = re.search(regex_version, file.read(), re.MULTILINE).group(1)

with open("README.md", "rb") as file:
    readme = file.read().decode("utf-8")


setup(
    name="fourcats-flask",
    version=version,
    packages=find_packages(),
    description="A flask encapsulated based on personal habits for fast use.",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="ShiWeiDong",
    author_email="shiweidong1993@gmail.com",
    url="https://github.com/FourCats-Py/FourCats-Flask",
    download_url="https://github.com/FourCats-Py/FourCats-Flask/archive/{}.tar.gz".format(version),
    keywords=["fourcats", "flask"],
    install_requires=[
        "flask-restx>=1.2.0", "Flask>=2.3.0", "loguru>=0.6.0", "PyYAML>=6.0", "Flask-HTTPAuth>=4.7.0",
        "mergedict>=1.0.0", "urllib3>=1.26.10", "SQLAlchemy>=2.0.0",  "Flask-SQLAlchemy>=3.0.0", "Flask-Cors>=3.0.10",
        "PyJWT>=2.4.0", "fourcats-utils>=0.0.4", "six", "certifi"
    ],
    python_requires=">=3.8"
)
